@@ -8,6 +8,10 @@
 #include <string.h>
 #include <stdlib.h>
 
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include "suck_config.h"
 #include "suck.h"
 #include "both.h"
@@ -22,7 +26,8 @@
 void chkhistory(PMaster master) {
 	FILE *fhist;
 	PList *array, ptr, curr, prev, tptr;
-	int i=0, nrfound=0;
+	int nrin=0, nrfound=0;
+	long valinhist = 0, nrinhist = 0;
 	char *cptr, linein[MAXLINLEN+1];
 	
 	if((fhist = fopen(master->history_file, "r")) == NULL) {
@@ -30,7 +35,8 @@ void chkhistory(PMaster master) {
 	}
 	else {	
 		if(master->debug == TRUE) {
-			do_debug("Chking %d items - master->nritems\nReading history file - %s\n", master->nritems, master->history_file);
+			do_debug("Chking %d master->nritems\nReading history file - %s\n",
+				 master->nritems, master->history_file);
 		}
 		print_phrases(master->msgs, chkh_phrases[1], NULL);
 		fflush(master->msgs);	/* so msg gets printed */
@@ -43,29 +49,33 @@ void chkhistory(PMaster master) {
 		}
 		else {
 			ptr = master->head;
-			i = 0;
+			nrin = 0;
 			while(ptr != NULL) {
-				array[i] = ptr;
-				i++;
+				array[nrin] = ptr;
+				nrin++;
 				ptr = ptr->next;
 			}
-		
+			if(master->debug == TRUE) {
+				do_debug("Added %d items to array for ssort, master->nritems = %d\n", nrin, master->nritems);
+			}
 			/* step 2, sort em */
-			ssort(array, master->nritems, 1); /* start with depth of 1 cause all start with < */
+			ssort(array, nrin, 1); /* start with depth of 1 cause all start with < */
 
 			/* step 3, find and mark dupes */
 			/* history file can have two valid types of lines */
 			/* the ones we are interested in is the ones that start with < */
 			/* we ignore the ones that start with [  and report the rest */
 			while(fgets(linein, MAXLINLEN, fhist) != NULL) {
+				nrinhist++;
 				if(linein[0] == '<') {
 					/* make sure we have valid msgid and null terminate it */
 					if((cptr = strchr(linein, '>'))  == NULL) {
 						error_log(ERRLOG_REPORT, chkh_phrases[2], linein, NULL);
 					}
 					else {
+						valinhist++;
 						*(cptr+1) = '\0';
-						if((tptr = my_bsearch(array, linein, master->nritems)) != NULL) {
+						if((tptr = my_bsearch(array, linein, nrin)) != NULL) {
 							/* found in our array flag for deletion */
 							/* it returns a PList * */
 							tptr->delete = TRUE;
@@ -74,8 +84,13 @@ void chkhistory(PMaster master) {
 					}
 				}
 			}
+			if(ferror(fhist) != 0) {
+				error_log(ERRLOG_REPORT, chkh_phrases[6], master->history_file, NULL);
+				clearerr(fhist);
+			}
+			
 			if(master->debug == TRUE) {
-				do_debug("%d duplicates found\n", nrfound);
+				do_debug("%d lines in history, %d valid lines, %d duplicates found\n", nrinhist, valinhist, nrfound);
 			}
 			/* step 4, delete em */
 			curr = master->head;
