@@ -29,6 +29,10 @@
 #include <errno.h>
 #include <signal.h>
 
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include "suck_config.h"
 #include "both.h"
 #include "phrases.h"
@@ -213,8 +217,13 @@ int move_msgs(PMaster master) {
 
 	/* which type of link will we create, symbolic or hard? Point to the right function */
 	/* this won't get used if link_type is NONE, so we don't handle that case */
+#ifdef EMX /* OS-2 doesn't have link or symlink */
+	linkit = NULL;
+	master->link_type = LINK_NONE;
+#else
 	linkit = (master->link_type == LINK_SYM) ? symlink : link;
-
+#endif
+	
 	if((dptr = opendir(master->msgdir)) == NULL) {
 		MyPerror(master->msgdir);
 		retval = RETVAL_ERROR;
