@@ -26,6 +26,10 @@
 #include <sys/param.h>
 #include <ctype.h>
 
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include "suck_config.h"
 #include "suck.h"
 #include "both.h"
@@ -250,7 +254,7 @@ int qcmp_msgid(const char *nr1, const char *nr2) {
 /*------------------------------------------------------------------------------*/
 int cmp_msgid(const char *nr1, const char *nr2) {
 	/* cmp two article id numbers */
-	/* which is <xxxxxxxxxxxxxxxxx@place.org */
+	/* which is <xxxxxxxxxxxxxxxxx@place.org> */
 	/* use case sensitive compares up to the @ */
 	/* then use case insensitive compares on the organization */
 	/* this is to conform to rfc822 */
@@ -293,10 +297,17 @@ int move_file(const char *src, const char *dest) {
 		retval = RETVAL_ERROR;
 	}
 	else if(rename(src, dest) != 0) {
+#ifdef EMX
+		if(errno != EACCES) {
+			MyPerror(src);
+			retval = RETVAL_ERROR;
+		}
+#else
 		if(errno != EXDEV) {
 			MyPerror(src);
 			retval = RETVAL_ERROR;
 		}
+#endif
 		else {
 			/* can't rename, try to copy file */
 			if((fpi = fopen(src, "r")) == NULL) {
