@@ -23,6 +23,10 @@
 #include <unistd.h>
 #endif
 
+#ifdef DMALLOC
+#include <dmalloc.h>
+#endif
+
 /*internal functions */
 #ifdef HAVE_GETTIMEOFDAY
 double get_elapsed(struct timeval *);
