@@ -1,7 +1,8 @@
 #ifndef SUCK_XOVER_H
 
 /*-----------------------prototypes */
-int do_group_xover(PMaster, char *, long);
+int do_group_xover(PMaster, char *, long, long);
+int get_xover(PMaster, char *, long, long);
 void get_xoverview(PMaster);
 
 #endif /* SUCK_XOVER_H */
